@@ -10,6 +10,7 @@ from typing import Any, Dict, Type
 from .base import DatabaseAdapter
 from .connection_parser import ConnectionParser
 from .exceptions import UnsupportedDatabaseError
+from .mongodb import MongoDBAdapter
 from .mysql import MySQLAdapter
 from .postgresql import PostgreSQLAdapter
 from .sqlite import SQLiteAdapter
@@ -35,6 +36,7 @@ class AdapterFactory:
             "mysql": MySQLAdapter,
             "sqlite": SQLiteEnterpriseAdapter,  # Use enterprise SQLite by default
             "sqlite_basic": SQLiteAdapter,  # Keep basic version available
+            "mongodb": MongoDBAdapter,  # MongoDB document database (v0.6.0+)
         }
 
     def register_adapter(
@@ -83,16 +85,21 @@ class AdapterFactory:
             scheme = components.get("scheme", "").lower()
 
             # Handle scheme variants
-            if scheme in ["postgres", "postgresql"]:
+            if scheme in ["postgres", "postgresql"] or scheme.startswith("postgresql+"):
                 return "postgresql"
             elif scheme.startswith("mysql"):
                 return "mysql"
-            elif scheme == "sqlite":
+            elif scheme.startswith("sqlite"):
                 return "sqlite"
+            elif scheme.startswith("mongodb"):
+                return "mongodb"
             elif scheme in self._adapters:
                 return scheme
             else:
                 raise UnsupportedDatabaseError(f"Unsupported database type: {scheme}")
+        except UnsupportedDatabaseError:
+            # Re-raise UnsupportedDatabaseError unchanged
+            raise
         except Exception as e:
             raise UnsupportedDatabaseError(f"Invalid connection string: {e}")
 
