@@ -44,14 +44,27 @@ class PostgreSQLAdapter(DatabaseAdapter):
         await self.close_connection_pool()
 
     async def create_connection_pool(self) -> None:
-        """Create PostgreSQL connection pool using asyncpg."""
+        """Create PostgreSQL connection pool using asyncpg with connection reset callback."""
         try:
             import asyncpg
 
+            # Define connection reset callback to prevent transaction leaks
+            async def reset_connection(conn):
+                """Reset connection state before returning to pool."""
+                try:
+                    if conn.is_in_transaction():
+                        logger.warning(
+                            "Connection released with open transaction - rolling back"
+                        )
+                        await conn.execute("ROLLBACK")
+                except Exception as e:
+                    logger.warning(f"Connection reset failed: {e}")
+
             # Build connection parameters
             params = self.get_connection_parameters()
 
-            # Create connection pool
+            # Create connection pool with reset callback
+            params["reset"] = reset_connection
             self.connection_pool = await asyncpg.create_pool(**params)
             self.is_connected = True
 
@@ -396,12 +409,14 @@ class PostgreSQLAdapter(DatabaseAdapter):
 
 
 class PostgreSQLTransaction:
-    """PostgreSQL transaction context manager."""
+    """PostgreSQL transaction context manager with guaranteed cleanup."""
 
     def __init__(self, connection_pool):
         self.connection_pool = connection_pool
         self.connection = None
         self.transaction = None
+        self._committed = False
+        self._rolled_back = False
 
     async def __aenter__(self):
         """Enter transaction context."""
@@ -411,13 +426,42 @@ class PostgreSQLTransaction:
         return self
 
     async def __aexit__(self, exc_type, exc_val, exc_tb):
-        """Exit transaction context."""
-        if exc_type is None:
-            # No exception, commit transaction
-            await self.transaction.commit()
-        else:
-            # Exception occurred, rollback transaction
-            await self.transaction.rollback()
+        """Exit transaction context with guaranteed cleanup."""
+        try:
+            if exc_type is None:
+                # No exception, commit transaction if not already done
+                if not self._committed and not self._rolled_back:
+                    await self.transaction.commit()
+                    self._committed = True
+            else:
+                # Exception occurred, rollback transaction if not already done
+                if not self._committed and not self._rolled_back:
+                    await self.transaction.rollback()
+                    self._rolled_back = True
+        except Exception as cleanup_error:
+            # Log cleanup error but don't raise (preserve original exception)
+            logger.error(f"Transaction cleanup failed: {cleanup_error}", exc_info=True)
+        finally:
+            # CRITICAL: Always release connection to pool
+            if self.connection:
+                await self.connection_pool.release(self.connection)
 
-        # Release connection back to pool
-        await self.connection_pool.release(self.connection)
+    async def commit(self):
+        """Explicitly commit transaction."""
+        if self._committed:
+            raise TransactionError("Transaction already committed")
+        if self._rolled_back:
+            raise TransactionError("Transaction already rolled back")
+
+        await self.transaction.commit()
+        self._committed = True
+
+    async def rollback(self):
+        """Explicitly rollback transaction."""
+        if self._committed:
+            raise TransactionError("Transaction already committed")
+        if self._rolled_back:
+            raise TransactionError("Transaction already rolled back")
+
+        await self.transaction.rollback()
+        self._rolled_back = True
