@@ -270,6 +270,16 @@ class SQLiteAdapter(DatabaseAdapter):
             try:
                 yield conn
             finally:
+                # Reset connection state before returning to pool
+                try:
+                    if conn.in_transaction:
+                        logger.warning(
+                            "Connection released with open transaction - rolling back"
+                        )
+                        await conn.rollback()
+                except Exception as e:
+                    logger.warning(f"Connection reset failed: {e}")
+
                 # Return connection to pool
                 async with self._pool_lock:
                     self._connection_pool.append(conn)
@@ -806,11 +816,13 @@ class SQLiteAdapter(DatabaseAdapter):
 
 
 class SQLiteTransaction:
-    """SQLite transaction context manager."""
+    """SQLite transaction context manager with guaranteed cleanup."""
 
     def __init__(self, adapter):
         self.adapter = adapter
         self.connection = None
+        self._committed = False
+        self._rolled_back = False
 
     async def __aenter__(self):
         """Enter transaction context."""
@@ -834,20 +846,54 @@ class SQLiteTransaction:
         return self
 
     async def __aexit__(self, exc_type, exc_val, exc_tb):
-        """Exit transaction context."""
+        """Exit transaction context with guaranteed cleanup."""
         try:
             if exc_type is None:
-                # No exception, commit transaction
-                await self.connection.commit()
+                # No exception, commit transaction if not already done
+                if not self._committed and not self._rolled_back:
+                    await self.connection.commit()
+                    self._committed = True
             else:
-                # Exception occurred, rollback transaction
-                await self.connection.rollback()
+                # Exception occurred, rollback transaction if not already done
+                if not self._committed and not self._rolled_back:
+                    await self.connection.rollback()
+                    self._rolled_back = True
+        except Exception as cleanup_error:
+            # Log cleanup error but don't raise (preserve original exception)
+            logger.error(
+                f"SQLite transaction cleanup failed: {cleanup_error}", exc_info=True
+            )
         finally:
-            # Return connection to pool
-            async with self.adapter._pool_lock:
-                self.adapter._connection_pool.append(self.connection)
-                self.adapter._pool_stats.idle_connections += 1
-                self.adapter._pool_stats.active_connections -= 1
+            # CRITICAL: Always return connection to pool
+            try:
+                async with self.adapter._pool_lock:
+                    self.adapter._connection_pool.append(self.connection)
+                    self.adapter._pool_stats.idle_connections += 1
+                    self.adapter._pool_stats.active_connections -= 1
+            except Exception as pool_error:
+                logger.error(
+                    f"Failed to return connection to pool: {pool_error}", exc_info=True
+                )
 
         # Return False to propagate exceptions
         return False
+
+    async def commit(self):
+        """Explicitly commit transaction."""
+        if self._committed:
+            raise Exception("Transaction already committed")
+        if self._rolled_back:
+            raise Exception("Transaction already rolled back")
+
+        await self.connection.commit()
+        self._committed = True
+
+    async def rollback(self):
+        """Explicitly rollback transaction."""
+        if self._committed:
+            raise Exception("Transaction already committed")
+        if self._rolled_back:
+            raise Exception("Transaction already rolled back")
+
+        await self.connection.rollback()
+        self._rolled_back = True
