@@ -263,7 +263,9 @@ class BulkOperations:
 
         # Perform actual database insertion
         try:
-            connection_string = self.dataflow.config.database.url or ":memory:"
+            connection_string = self.dataflow.config.database.get_connection_url(
+                self.dataflow.config.environment
+            )
             database_type = self.dataflow._detect_database_type()
             table_name = self.dataflow._class_name_to_table_name(model_name)
 
@@ -450,7 +452,9 @@ class BulkOperations:
                 )
 
                 # Get database connection and execute UPDATE
-                connection_string = self.dataflow.config.database.url or ":memory:"
+                connection_string = self.dataflow.config.database.get_connection_url(
+                    self.dataflow.config.environment
+                )
                 database_type = self.dataflow._detect_database_type()
                 table_name = self.dataflow._class_name_to_table_name(model_name)
 
@@ -722,7 +726,9 @@ class BulkOperations:
             logger.warning("BULK_DELETE: Processing filter-based delete")
             try:
                 # Get database connection and execute DELETE
-                connection_string = self.dataflow.config.database.url or ":memory:"
+                connection_string = self.dataflow.config.database.get_connection_url(
+                    self.dataflow.config.environment
+                )
                 database_type = self.dataflow._detect_database_type()
                 table_name = self.dataflow._class_name_to_table_name(model_name)
 
@@ -895,7 +901,9 @@ class BulkOperations:
 
         # Perform actual database upsert
         try:
-            connection_string = self.dataflow.config.database.url or ":memory:"
+            connection_string = self.dataflow.config.database.get_connection_url(
+                self.dataflow.config.environment
+            )
             database_type = self.dataflow._detect_database_type()
             table_name = self.dataflow._class_name_to_table_name(model_name)
 
