@@ -433,7 +433,7 @@ class TableRenameAnalyzer:
             rc.delete_rule,
             rc.update_rule,
             pg_get_constraintdef(pgc.oid) as constraint_definition
-        FROM information_schema.table_constraints AS tc 
+        FROM information_schema.table_constraints AS tc
         JOIN information_schema.key_column_usage AS kcu
             ON tc.constraint_name = kcu.constraint_name
             AND tc.table_schema = kcu.table_schema
@@ -476,7 +476,11 @@ class TableRenameAnalyzer:
 
         except Exception as e:
             self.logger.error(f"Error finding FK references: {e}")
-            return []
+            raise RuntimeError(
+                f"Failed to discover foreign key constraints for table '{table_name}'. "
+                f"Cannot safely proceed with rename operation without complete FK information. "
+                f"Error: {e}"
+            )
 
     async def find_outgoing_foreign_key_references(
         self, table_name: str, connection: Optional[asyncpg.Connection] = None
@@ -496,7 +500,7 @@ class TableRenameAnalyzer:
             rc.delete_rule,
             rc.update_rule,
             pg_get_constraintdef(pgc.oid) as constraint_definition
-        FROM information_schema.table_constraints AS tc 
+        FROM information_schema.table_constraints AS tc
         JOIN information_schema.key_column_usage AS kcu
             ON tc.constraint_name = kcu.constraint_name
             AND tc.table_schema = kcu.table_schema
@@ -543,7 +547,11 @@ class TableRenameAnalyzer:
 
         except Exception as e:
             self.logger.error(f"Error finding outgoing FK references: {e}")
-            return []
+            raise RuntimeError(
+                f"Failed to discover outgoing foreign key constraints for table '{table_name}'. "
+                f"Cannot safely proceed with rename operation without complete FK information. "
+                f"Error: {e}"
+            )
 
     async def find_view_dependencies(
         self, table_name: str, connection: Optional[asyncpg.Connection] = None
@@ -554,12 +562,12 @@ class TableRenameAnalyzer:
 
         # Query for regular views - improved pattern matching
         view_query = """
-        SELECT 
+        SELECT
             viewname,
             definition,
             schemaname,
             false as is_materialized
-        FROM pg_views 
+        FROM pg_views
         WHERE schemaname = 'public'
             AND (
                 definition ILIKE '%FROM ' || $1 || ' %'
@@ -568,10 +576,10 @@ class TableRenameAnalyzer:
                 OR definition ILIKE '%JOIN ' || $1 || '\n%'
                 OR definition ILIKE '%' || $1 || '%'
             )
-        
+
         UNION ALL
-        
-        SELECT 
+
+        SELECT
             matviewname as viewname,
             definition,
             schemaname,
@@ -617,7 +625,11 @@ class TableRenameAnalyzer:
 
         except Exception as e:
             self.logger.error(f"Error finding view dependencies: {e}")
-            return []
+            raise RuntimeError(
+                f"Failed to discover view dependencies for table '{table_name}'. "
+                f"Cannot safely proceed with rename operation without complete dependency information. "
+                f"Error: {e}"
+            )
 
     async def find_index_dependencies(
         self, table_name: str, connection: Optional[asyncpg.Connection] = None
@@ -627,12 +639,12 @@ class TableRenameAnalyzer:
             connection = await self._get_connection()
 
         index_query = """
-        SELECT 
+        SELECT
             indexname,
             tablename,
             indexdef
-        FROM pg_indexes 
-        WHERE tablename = $1 
+        FROM pg_indexes
+        WHERE tablename = $1
             AND schemaname = 'public'
         ORDER BY indexname
         """
@@ -661,7 +673,11 @@ class TableRenameAnalyzer:
 
         except Exception as e:
             self.logger.error(f"Error finding index dependencies: {e}")
-            return []
+            raise RuntimeError(
+                f"Failed to discover index dependencies for table '{table_name}'. "
+                f"Cannot safely proceed with rename operation without complete dependency information. "
+                f"Error: {e}"
+            )
 
     async def find_trigger_dependencies(
         self, table_name: str, connection: Optional[asyncpg.Connection] = None
@@ -677,7 +693,7 @@ class TableRenameAnalyzer:
             action_timing,
             action_statement,
             event_object_table as table_name
-        FROM information_schema.triggers 
+        FROM information_schema.triggers
         WHERE event_object_table = $1
             AND event_object_schema = 'public'
         ORDER BY trigger_name
@@ -702,7 +718,11 @@ class TableRenameAnalyzer:
 
         except Exception as e:
             self.logger.error(f"Error finding trigger dependencies: {e}")
-            return []
+            raise RuntimeError(
+                f"Failed to discover trigger dependencies for table '{table_name}'. "
+                f"Cannot safely proceed with rename operation without complete dependency information. "
+                f"Error: {e}"
+            )
 
     async def build_dependency_graph(
         self, root_table: str, schema_objects: List[SchemaObject]
